VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cLight"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

Option Explicit

' the format direct3d expects lights in
Private Lgt As D3DLIGHT8

Public Index As Integer

' a user friendly name for the Lgt
Public Tag As String

' tracks whether light is on/off
Private Enable As Boolean

' the frame the light is attached to, if any
Public Frame As cFrame







' increment count
Private Sub Class_Initialize()
On Error Resume Next
    Index = UBound(modGeometry.Light)
    SetAmbient 1, 1, 1, 1
    SetDiffuse 1, 1, 1, 1
    SetSpecular 1, 1, 1, 1
    With Light
        .Type = D3DLIGHT_POINT
        .Range = 100
    End With
End Sub



' get/set light
Property Get Light() As D3DLIGHT8
    Light = Lgt
End Property
Property Let Light(newLight As D3DLIGHT8)
    Lgt = newLight
End Property



' set light type
Sub SetLightType(LightType As CONST_D3DLIGHTTYPE)
On Error Resume Next
    Lgt.Type = LightType
End Sub



' set ambient color
Sub SetAmbient(a As Single, r As Single, g As Single, b As Single)
On Error Resume Next
    With Lgt.Ambient
        .a = a
        .r = r
        .g = g
        .b = b
    End With
End Sub



' set diffuse color
Sub SetDiffuse(a As Single, r As Single, g As Single, b As Single)
On Error Resume Next
    With Lgt.Diffuse
        .a = a
        .r = r
        .g = g
        .b = b
    End With
End Sub



' set specular color
Sub SetSpecular(a As Single, r As Single, g As Single, b As Single)
On Error Resume Next
    With Lgt.Specular
        .a = a
        .r = r
        .g = g
        .b = b
    End With
End Sub



' move the light
Sub Move(Translation As D3DVECTOR)
On Error Resume Next
    D3DXVec3Add Lgt.Position, Lgt.Position, Translation
End Sub



' set the position of the light
Sub SetPosition(newPosition As D3DVECTOR)
On Error Resume Next
    Lgt.Position = newPosition
End Sub



' rotate the light
Sub Rotate(Rotation As D3DVECTOR)
On Error Resume Next
    D3DXVec3Add Lgt.Direction, Lgt.Direction, Rotation
End Sub



' set the direction of the light
Sub SetDirection(newDirection As D3DVECTOR)
On Error Resume Next
    Lgt.Direction = newDirection
End Sub



' set attenuation
Sub SetAttentuation(Att0 As Single, Att1 As Single, Att2 As Single)
On Error Resume Next
    With Lgt
        .Attenuation0 = Att0
        .Attenuation1 = Att1
        .Attenuation2 = Att2
    End With
End Sub



' set range
Sub SetRange(newRange As Single)
On Error Resume Next
    Lgt.Range = newRange
End Sub



' set fall off
Sub SetFalloff(Falloff As Single)
On Error Resume Next
    Lgt.Falloff = Falloff
End Sub


' set cone angles
Sub SetConeAngles(theta As Single, phi As Single)
On Error Resume Next
    Lgt.theta = theta
    Lgt.phi = phi
End Sub



' create a directional light
Sub SetToDirectionalLight(Direction As D3DVECTOR, Ambient As D3DCOLORVALUE, Diffuse As D3DCOLORVALUE, Specular As D3DCOLORVALUE)
On Error Resume Next
    With Lgt
        .Type = D3DLIGHT_DIRECTIONAL
        .Direction = Direction
        .Ambient = Ambient
        .Diffuse = Diffuse
        .Specular = Specular
    End With
End Sub



' create a point light
Sub SetToPointLight(Position As D3DVECTOR, Ambient As D3DCOLORVALUE, Diffuse As D3DCOLORVALUE, Specular As D3DCOLORVALUE, Attenuation0 As Single, Attenuation1 As Single, Attenuation2 As Single, Range As Single)
On Error Resume Next
    With Lgt
        .Type = D3DLIGHT_POINT
        .Position = Position
        .Ambient = Ambient
        .Diffuse = Diffuse
        .Specular = Specular
        .Attenuation0 = Attenuation0
        .Attenuation1 = Attenuation1
        .Attenuation2 = Attenuation2
        .Range = Range
    End With
End Sub



' create a spot light
Sub SetToSpotLight(Position As D3DVECTOR, Direction As D3DVECTOR, Ambient As D3DCOLORVALUE, Diffuse As D3DCOLORVALUE, Specular As D3DCOLORVALUE, theta As Single, phi As Single, Attenuation0 As Single, Attenuation1 As Single, Attenuation2 As Single, Range As Single, Falloff As Single)
On Error Resume Next
    With Lgt
        .Type = D3DLIGHT_SPOT
        .Position = Position
        .Direction = Direction
        .Ambient = Ambient
        .Diffuse = Diffuse
        .Specular = Specular
        .theta = theta
        .phi = phi
        .Attenuation0 = Attenuation0
        .Attenuation1 = Attenuation1
        .Attenuation2 = Attenuation2
        .Range = Range
    End With
End Sub



' tell direct3d to add light
Sub Load()
On Error Resume Next
    D3Ddevice.SetLight Index, Lgt
End Sub



' turn light on/off
Property Get Enabled() As Boolean
On Error Resume Next
    Enabled = Enable
End Property
Property Let Enabled(bEnabled As Boolean)
On Error Resume Next
    Enable = bEnabled
    D3Ddevice.LightEnable Index, bEnabled
End Property


