VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cLocation"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' the parent frame object this location belongs to
Public Parent As cFrame

' the origin represents a translation applied to move an object
Dim Orig As D3DVECTOR

' the orientation represents a rotation applied to rotate an object
Dim Orient As D3DVECTOR

' the scale of the object represents a resizing applied to size an object
Dim Siz As D3DVECTOR

' the velocity of the object represents the rate of movement
Dim Vel As D3DVECTOR

' the spin of the object represents the rate of rotation
Dim Spn As D3DVECTOR

' the growth of the object represents the rate of change in size
Dim Grow As D3DVECTOR

' the tranformation of this location to the world space origin
Dim matWorldSpace As D3DMATRIX

' the length of time since the last refresh
Dim timeLast As Single



Private Sub Class_Initialize()
    Siz = Vector3(1, 1, 1)
End Sub



Property Get Origin() As D3DVECTOR
    Origin = Orig
End Property
Property Let Origin(newOrigin As D3DVECTOR)
    Orig = newOrigin
End Property

Property Get Orientation() As D3DVECTOR
    Orientation = Orient
End Property
Property Let Orientation(newOrientation As D3DVECTOR)
    Orient = newOrientation
End Property

Property Get Size() As D3DVECTOR
    Size = Siz
End Property
Property Let Size(newSize As D3DVECTOR)
    Siz = newSize
End Property

Property Get Velocity() As D3DVECTOR
    Velocity = Vel
End Property
Property Let Velocity(newVelocity As D3DVECTOR)
    Vel = newVelocity
End Property

Property Get Spin() As D3DVECTOR
    Spin = Spn
End Property
Property Let Spin(newSpin As D3DVECTOR)
    Spn = newSpin
End Property

Property Get Growth() As D3DVECTOR
    Growth = Grow
End Property
Property Let Growth(newGrowth As D3DVECTOR)
    Grow = newGrowth
End Property

Sub GetWorldSpaceMatrix(matTemp As D3DMATRIX)
    matTemp = matWorldSpace
End Sub

Property Get TimeSinceLastRefresh() As Single
    TimeSinceLastRefresh = timeLast
End Property






' this function transforms the frame using the origin, orientation, size
' velocity, spin, growth time elapsed
' you must feed it the amount of time for the move in seconds
Sub Refresh(timeElapsed As Single)
Dim l As Long
Dim vec As D3DVECTOR
On Error Resume Next
    ' translate origin by velocity * time
    D3DXVec3Scale vec, Vel, timeElapsed
    D3DXVec3Add Orig, Orig, vec
    ' rotate orientation by rotation * time
    D3DXVec3Scale vec, Spn, timeElapsed
    D3DXVec3Add Orient, Orient, vec
    ' grow size by growth * time
    D3DXVec3Scale vec, Grow, timeElapsed
    D3DXVec3Add Siz, Siz, vec
    ' cache time
    timeLast = timeElapsed
End Sub



' this function undoes a refresh, or reverses the animation
Sub UndoRefresh()
Dim l As Long
Dim vec As D3DVECTOR
On Error Resume Next
    ' translate origin by velocity * time
    D3DXVec3Scale vec, Vel, timeLast
    D3DXVec3Subtract Orig, Orig, vec
    ' rotate orientation by rotation * time
    D3DXVec3Scale vec, Spn, timeLast
    D3DXVec3Subtract Orient, Orient, vec
    ' grow size by growth * time
    D3DXVec3Scale vec, Grow, timeLast
    D3DXVec3Subtract Siz, Siz, vec
End Sub



' sets the location as the centre of the world
Sub SetAsCurrent()
On Error Resume Next
Dim matWorld As D3DMATRIX
Dim matTrans As D3DMATRIX, matRot As D3DMATRIX, matScaling As D3DMATRIX
    ' build translation matrix
    D3DXMatrixTranslation matTrans, Orig.x, Orig.y, Orig.z
    ' build rotation matrix
    D3DXMatrixRotationYawPitchRoll matRot, Orient.y, Orient.x, Orient.z
    ' build scaling matrix
    D3DXMatrixScaling matScaling, Siz.x, Siz.y, Siz.z
    ' combine them and update the direct3d world matrix
    D3DXMatrixMultiply matTrans, matTrans, matScaling
    D3DXMatrixMultiply matWorld, matRot, matTrans
    D3Ddevice.MultiplyTransform D3DTS_WORLD, matWorld
    ' remember matrix to transform back
    D3Ddevice.GetTransform D3DTS_WORLD, matWorldSpace
    D3DXMatrixInverse matWorldSpace, 0, matWorldSpace
End Sub



' sets the locations data in one function call
Sub SetVectors(Origin As D3DVECTOR, Orientation As D3DVECTOR, Size As D3DVECTOR, Velocity As D3DVECTOR, Spin As D3DVECTOR, Growth As D3DVECTOR)
On Error Resume Next
    With Me
        Set .Parent = Me
        .Origin = Origin
        .Orientation = Orientation
        .Size = Size
        .Velocity = Velocity
        .Spin = Spin
        .Growth = Growth
    End With
End Sub
