Attribute VB_Name = "modCollision"
Option Explicit

' for copying linear chunks of memory, well fast
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

' a collision test result stores 2 locations, a face, a distance, and a 2D vectors
Type CollisionDescription
    Location1 As cLocation ' currently not used
    Location2 As cLocation
    FaceIndex As Long
    Distance As Single
    Coord As D3DVECTOR2
End Type

' temporary arrary of collision detection test results
Public tmpResults() As CollisionDescription


' the function tests every frame in every location with a moving
' point, to see if it has collided with any of them, then it
' reports back the results in an array

Function CollisionTestAllLocationsWithPoint(Point As D3DVECTOR, Dir As D3DVECTOR) As Boolean
Dim l As Long
On Error Resume Next
    ReDim tmpResults(0)
    CollisionTestFrameAndChildrenWithPoint WorldFrame, 1, Point, Dir
    CollisionTestAllLocationsWithPoint = UBound(tmpResults)
End Function

' this is the recursive collision detection function, which
' recursively cycles through every location below the initial
' one called. it uses a sphere collision test before going to
' test every triangle to save time, by quickly testing for
' locations that are too far apart

Sub CollisionTestFrameAndChildrenWithPoint(Frame As cFrame, LocationIndex As Long, Point As D3DVECTOR, Dir As D3DVECTOR)
Dim matTransform As D3DMATRIX
Dim matTransposeInvTransform As D3DMATRIX
Dim Origin As D3DVECTOR
Dim Ray As D3DVECTOR
Dim vec As D3DVECTOR
Dim DistSq As Single
Dim MaxDistSq As Single
Dim Hit As Long
Dim FaceIndex As Long
Dim HitU As Single, HitV As Single
Dim HitDist As Single
Dim Location As cLocation
Dim l As Long
On Error Resume Next
    If LocationIndex > Frame.NumLocations Then Exit Sub
    Set Location = Frame.Location(LocationIndex)
    ' transform point and ray into location space
    Location.GetWorldSpaceMatrix matTransform
    D3DXVec3TransformCoord Origin, Point, matTransform
    D3DXMatrixInverse matTransposeInvTransform, 0, matTransform
    D3DXMatrixTranspose matTransposeInvTransform, matTransposeInvTransform
    D3DXVec3TransformNormal Ray, Dir, matTransposeInvTransform
    ' test using spheres (this is the spheretest function written inline)
    MaxDistSq = D3DXVec3Length(Ray) + Frame.Radius
    MaxDistSq = MaxDistSq * MaxDistSq
    DistSq = D3DXVec3LengthSq(Origin)
    ' test if out of range
    If DistSq < MaxDistSq Then
        ' in range, now try full test
        Hit = 0
        D3DX.Intersect Location.Parent.Mesh, Origin, Ray, Hit, FaceIndex, HitU, HitV, HitDist
'        If Not Hit Then ' try the other way round
'            D3DXVec3Add Origin, Origin, Ray
'            Ray.x = -Ray.x
'            Ray.y = -Ray.y
'            Ray.z = -Ray.z
'            D3DX.Intersect Location.Parent.Mesh, Origin1, Ray, Hit, FaceIndex, HitU, HitV, HitDist
'        End If
        If Hit Then
            If HitDist < 1 Then 'RayLength Then
            ' now report collision result
            ReDim Preserve tmpResults(0 To UBound(tmpResults) + 1)
            With tmpResults(UBound(tmpResults))
                Set .Location2 = Location
                .FaceIndex = FaceIndex
                .Distance = HitDist
                .Coord = Vector2(HitU, HitV)
            End With
            End If
        End If
    End If
    ' recursively call children
    If Frame.NumChildren Then
        For l = 1 To Frame.NumChildren
            CollisionTestFrameAndChildrenWithPoint Frame.ChildFrame(l), 1, Point, Dir
        Next
    End If
    CollisionTestFrameAndChildrenWithPoint Frame, LocationIndex + 1, Point, Dir
End Sub






' THE FOLLOWING FUNCTIONS ARE STILL UNDER DEVELOPMENT, THEY ARE
' SLOW AND BUGGY AT THE MOMENT. EVENTUALLY THEY WILL BE ABLE TO
' TEST EVERY OBJECT IN THE ENGINE FOR COLLISION, AND MAYBE EVEN
' APPLY SOME CRASHING BOUNCING BASHING PHYSICS SORTA STUFF





'Sub AddResult(newResult As CollisionDescription)
'On Error Resume Next
'    ReDim Preserve tmpResults(0 To UBound(tempresults) + 1)
'    tmpResults(UBound(tmpResults)) = newResult
'End Sub
'
'Function CollisionTestAllPossibleLocations(Results() As CollisionDescription) As Boolean
'On Error Resume Next
'    ReDim tmpResults(0)
'    CollisionTestFrameAndChildren WorldFrame, 1
'    Results = tmpResults
'End Function
'
'Sub CollisionTestFrameAndChildren(Frame As cFrame, LocationIndex As Long)
'Dim l As Long
'Dim m As Long
'Static matCurrent As D3DMATRIX
'On Error Resume Next
'    If LocationIndex > Frame.NumLocations Then Exit Sub
'    If Frame.NumLocations Then
'        For l = 1 To Frame.NumLocations
'            If Frame.NumCollisionTests Then
'                For m = 1 To Frame.NumCollisionTests
'                    CollisionTestLocationAgainstFrame Frame.Location(LocationIndex), Frame.CollisionTest(m)
'                Next
'            End If
'        Next
'    End If
'    If Frame.NumChildren Then
'        For l = 1 To Frame.NumChildren
'            CollisionTestFrameAndChildren Frame.ChildFrame(l), 1
'        Next
'    End If
'    CollisionTestFrameAndChildren Frame, LocationIndex + 1
'End Sub
'
'Sub CollisionTestLocationAgainstFrame(Location As cLocation, Frame As cFrame)
'Dim Velocity As D3DVECTOR, Spin As D3DVECTOR, Growth As D3DVECTOR
'Dim timeElapsed As Single
'Dim matTransform As D3DMATRIX
'Dim matTrans As D3DMATRIX, matRot As D3DMATRIX, matScaling As D3DMATRIX
'Dim Origin1 As D3DVECTOR, Origin2 As D3DVECTOR
'Dim vec As D3DVECTOR
'Dim DistSq As Single
'Dim MaxDistSq As Single
'Dim Collision As CollisionDescription
'Dim VertexBuffer As Direct3DVertexBuffer8
'Dim pVertexBuffer As Long
'Dim FVF As CONST_D3DFVFFLAGS
'Dim VertexLength As Long
'Dim NumVertices As Long
'Dim Point(0) As D3DVECTOR
'Dim Ray As D3DVECTOR
'Dim Hit As Long
'Dim FaceIndex As Long
'Dim HitU As Single, HitV As Single
'Dim HitDist As Single
'Dim l As Long
'Dim m As Long
'On Error Resume Next
'    If Frame.NumLocations Then
''        Velocity = Location.Velocity
''        Spin = Location.Spin
''        Growth = Location.Growth
''        timeElapsed = Location.TimeSinceLastRefresh
''        D3DXVec3Scale vec, Velocity, timeElapsed
''        D3DXVec3Scale vec, Spin, timeElapsed
''        D3DXVec3Scale vec, Growth, timeElapsed
''        ' build translation matrix
''        D3DXMatrixTranslation matTrans, Velocity.x, Velocity.y, Velocity.z
''        ' build rotation matrix
''        D3DXMatrixRotationYawPitchRoll matRot, Spin.y, Spin.x, Spin.z
''        ' build scaling matrix
''        D3DXMatrixScaling matScaling, Growth.x, Growth.y, Growth.z
''        ' combine them
''        D3DXMatrixMultiply matTrans, matTrans, matScaling
''        D3DXMatrixMultiply matTransform, matRot, matTrans
''        ' find ray vector
''        D3DXVec3TransformCoord Ray, Ray, matTransform
''        Ray.x = -Ray.x: Ray.y = -Ray.y: Ray.z = -Ray.z
'        ' transform locations into world space
'        Origin1 = Location.Origin
'        D3DXVec3TransformCoord Origin1, Origin1, Location.WorldSpaceMatrix
'        For l = 1 To Frame.NumLocations
'            ' check the 2 locations aren't the same!
'            If Location Is Frame.Location(l) Then GoTo NextTest
'            ' transform locations into world space
'            Origin2 = Frame.Location(l).Origin
'            D3DXVec3TransformCoord Origin2, Origin2, Frame.Location(l).WorldSpaceMatrix
'            ' test using spheres (this is the spheretest function written inline)
'            MaxDistSq = Location.Parent.Radius + Frame.Radius
'            MaxDistSq = MaxDistSq * MaxDistSq
'            D3DXVec3Subtract vec, Origin2, Origin1
'            DistSq = D3DXVec3LengthSq(vec)
'            ' test if out of range
'            If DistSq < MaxDistSq Then
''                ' in range, so now to a full collision test
''                ' get vertices
''                Set VertexBuffer = Location.Parent.Mesh.GetVertexBuffer
''                FVF = Location.Parent.Mesh.GetFVF
''                VertexLength = D3DX.GetFVFVertexSize(FVF)
''                pVertexBuffer = Location.Parent.Mesh.LockVertexBuffer(D3DLOCK_READONLY)
''                NumVertices = Location.Parent.Mesh.GetNumVertices
''                For m = 1 To NumVertices
''                    DXCopyMemory Point(0), pVertexBuffer + (m - 1) * VertexLength, 36
''                    D3DX.Intersect Location.Parent.Mesh, Point(0), Ray, Hit, FaceIndex, HitU, HitV, HitDist
''                    If Hit Then
'                        ' now report collision result
'                        With Collision
'                            Set .Location1 = Location
'                            Set .Location2 = Frame.Location(l)
'                        End With
'                        ReDim Preserve tmpResults(0 To UBound(tmpResults) + 1)
'                        tmpResults(UBound(tmpResults)) = Collision
''                    End If
''                Next
'                Location.Parent.Mesh.UnlockVertexBuffer
'            End If
'NextTest:
'        Next
'    End If
'End Sub
'
'Function SphereTest(Origin1 As D3DVECTOR, Origin2 As D3DVECTOR, Radius1 As Single, Radius2 As Single) As Boolean
'Dim vec As D3DVECTOR
'Dim DistSq As Single
'Dim MaxDistSq As Single
'On Error Resume Next
'    MaxDistSq = Radius1 + Radius2
'    MaxDistSq = MaxDistSq * MaxDistSq
'    D3DXVec3Subtract vec, Origin2, Origin1
'    DistSq = D3DXVec3LengthSq(vec)
'    SphereTest = (DistSq < MaxDistSq)
'End Function
'
'
