Attribute VB_Name = "modGeometry"

Option Explicit

' the materials used
Public Material() As New cMaterial
' the textures used
Public Texture() As New cTexture
' the lights used
Public Light() As New cLight
' the cameras used
Public Camera() As New cCamera

' cached material pointer, speeds up rendering by decreasing material changes
Public lastMaterial As cMaterial
' cached texture pointer, speeds up rendering by decreasing texture changes
Public lastTexture As cTexture

' the root of all frame objects in the world!
Public WorldFrame As New cFrame

' important constant for geometry
Public Const PI As Single = 3.1415
Public Const PIby2 As Single = PI * 2








' resets the contents of the engine
Sub Reset()
On Error Resume Next
    ReDim Material(0)
    ReDim Texture(0)
    ReDim Light(0)
    ReDim Camera(0)
    ReDim Sound(0)
    WorldFrame.NumChildren = 0
End Sub



' adds a new camera
Function AddNewCamera() As cCamera
On Error Resume Next
    ReDim Preserve Camera(0 To UBound(Camera) + 1)
    Set AddNewCamera = Camera(UBound(Camera))
End Function

' adds a new light
Function AddNewLight() As cLight
On Error Resume Next
    ReDim Preserve Light(0 To UBound(Light) + 1)
    Set AddNewLight = Light(UBound(Light))
End Function

' adds a new material
Function AddNewMaterial() As cMaterial
On Error Resume Next
    ReDim Preserve Material(0 To UBound(Material) + 1)
    Set AddNewMaterial = Material(UBound(Material))
End Function

' adds a new texture
Function AddNewTexture() As cTexture
On Error Resume Next
    ReDim Preserve Texture(0 To UBound(Texture) + 1)
    Set AddNewTexture = Texture(UBound(Texture))
End Function

' adds a new sound
Function AddNewSound() As cSound
On Error Resume Next
    ReDim Preserve Sound(0 To UBound(Sound) + 1)
    Set AddNewSound = Light(UBound(Sound))
End Function



' creates a 2d vector
Function Vector2(x As Single, y As Single) As D3DVECTOR2
    With Vector2
        .x = x
        .y = y
    End With
End Function
' creates a 3d vector
Function Vector3(x As Single, y As Single, z As Single) As D3DVECTOR
    With Vector3
        .x = x
        .y = y
        .z = z
    End With
End Function
' creates a d3dvertex
Function MakeVertex(x As Single, y As Single, z As Single, nx As Single, ny As Single, nz As Single, tu As Single, tv As Single) As D3DVERTEX
    With MakeVertex
        .x = x
        .y = y
        .z = z
        .nx = nx
        .ny = ny
        .nz = nz
        .tu = tu
        .tv = tv
    End With
End Function

' creates a d3dcolorvalue
Function MakeD3DCOLORVALUE(a As Single, r As Single, g As Single, b As Single) As D3DCOLORVALUE
    With MakeD3DCOLORVALUE
        .a = a
        .r = r
        .g = g
        .b = b
    End With
End Function



' shifts the world matrix by a vector and rotates it by a vector and scales it by a vector
Sub MoveWorldMatrix2(Translation As D3DVECTOR, Rotation As D3DVECTOR, Scaling As D3DVECTOR)
Dim matWorld As D3DMATRIX
Dim matTrans As D3DMATRIX, matRot As D3DMATRIX, matScaling As D3DMATRIX
    ' build translation matrix
    D3DXMatrixTranslation matTrans, Translation.x, Translation.y, Translation.z
    ' build rotation matrix
    D3DXMatrixRotationYawPitchRoll matRot, Rotation.y, Rotation.x, Rotation.z
    ' build scaling matrix
    D3DXMatrixScaling matScaling, Scaling.x, Scaling.y, Scaling.z
    ' combine them and update the direct3d world matrix
    D3DXMatrixMultiply matTrans, matTrans, matScaling
    D3DXMatrixMultiply matWorld, matRot, matTrans
    D3Ddevice.MultiplyTransform D3DTS_WORLD, matWorld
End Sub



' shifts the world matrix in reverse
Sub MoveBackWorldMatrix(Translation As D3DVECTOR, Rotation As D3DVECTOR, Scaling As D3DVECTOR)
On Error Resume Next
    'MoveWorldMatrix Vector3(-Translation.x, -Translation.y, -Translation.z), Vector3(-Rotation.x, -Rotation.y, -Rotation.z), Vector3(1 / Scaling.x, 1 / Scaling.y, 1 / Scaling.z)
End Sub



' sets the world matrix
Sub SetWorldMatrix(Position As D3DVECTOR, Rotation As D3DVECTOR, Scaling As D3DVECTOR)
Dim matWorld As D3DMATRIX
Dim matTrans As D3DMATRIX, matRot As D3DMATRIX, matScaling As D3DMATRIX
    ' build translation matrix
    D3DXMatrixTranslation matTrans, Position.x, Position.y, Position.z
    ' build rotation matrix
    D3DXMatrixRotationYawPitchRoll matRot, Rotation.y, Rotation.x, Rotation.z
    ' build scaling matrix
    D3DXMatrixScaling matScaling, Scaling.x, Scaling.y, Scaling.z
    ' combine them and update the direct3d world matrix
    D3DXMatrixMultiply matWorld, matRot, matTrans
    D3DXMatrixMultiply matWorld, matWorld, matScaling
    D3Ddevice.SetTransform D3DTS_WORLD, matWorld
End Sub



' refreshes every location in the engine
Sub RefreshAllLocations(timeElapsed As Single)
Dim l As Long
On Error Resume Next
    Set lastMaterial = Nothing
    Set lastTexture = Nothing
    D3Ddevice.BeginScene
    RefreshAllFrameLocationsAndChildren WorldFrame, timeElapsed
    D3Ddevice.EndScene
End Sub



' recursive function - goes though each frame and refreshs all locations
Sub RefreshAllFrameLocationsAndChildren(Frame As cFrame, timeElapsed As Single)
Dim l As Long
Dim matTemp As D3DMATRIX
On Error Resume Next
    ' set location as centre of the world
    Frame.RefreshLocations timeElapsed
    ' render children
    If Frame.NumChildren Then
        For l = 1 To Frame.NumChildren
            RefreshAllFrameLocationsAndChildren Frame.ChildFrame(l), timeElapsed
        Next
    End If
End Sub
